# curtain.py
#
# Sample Webots controller file for driving a
# 'curtain' of actuated hanging chains.
#
# No copyright, 2020-2022, Garth Zeglin.  This file is
# explicitly placed in the public domain.

print("loading curtain.py...")

# Import the Webots simulator API.
from controller import Robot
from controller import Keyboard

# Import the standard Python math library.
import math

# Define the time step in milliseconds between
# controller updates.
EVENT_LOOP_DT = 20

################################################################

# Request a proxy object representing the robot to control.
robot = Robot()
name = robot.getName()
print(f"curtain.py waking up for {name}...")

# Query the number of devices.  The curtain.proto model has one joint actuator
# and one sensor per chain.
num_devices = robot.getNumberOfDevices()
chains = num_devices // 2
print(f"Found {num_devices} devices, assuming {chains} hanging chains.")

# Enable computer keyboard input for user control.
keyboard = Keyboard()
keyboard.enable(EVENT_LOOP_DT)

# Fetch handles for the joint sensors.  The names are generated by the curtain.proto scripting.
joints = [robot.getDevice('joint%d' % (jnum+1)) for jnum in range(chains)]

# Specify the sampling rate for the joint sensors.
for j in joints:
    j.enable(EVENT_LOOP_DT)

# Fetch handles for the position actuator at the top of each chain.
motors = [robot.getDevice('motor%d' % (jnum+1)) for jnum in range(chains)]
for m in motors:
    m.setPosition(0.0)

################################################################
# Run an event loop until the simulation quits,
# indicated by the step function returning -1.

while robot.step(EVENT_LOOP_DT) != -1:

    # Read simulator clock time.
    t = robot.getTime()

    # Read the new joint positions.
    q = [j.getValue() for j in joints]

    # Read any computer keyboard keypresses.  Returns -1 or an integer keycode while a key is held down.
    key = keyboard.getKey()
    if key != -1:
        # convert the integer key number to a lowercase single-character string        
        letter = chr(key).lower()

        if letter == 'q':
            for i, m in enumerate(motors):
                if i <= 3:
                    m.setPosition(-0.5)
                else:
                    m.setTorque(0)
                    
        elif letter == 'w':
            for i, m in enumerate(motors):
                if i > 3 and i <= 7:
                    m.setPosition(-0.5)
                else:
                    m.setTorque(0)
                    
        elif letter == 'e':
            for i, m in enumerate(motors):
                if i > 7 and i <= 11:
                    m.setPosition(-0.5)
                else:
                    m.setTorque(0)
                    
        elif letter == 'r':
            for i, m in enumerate(motors):
                if i > 11 and i <= 15:
                    m.setPosition(-0.5)
                else:
                    m.setTorque(0)
                    
        elif letter == 't':
            for i, m in enumerate(motors):
                if i > 15 and i <= 19:
                    m.setPosition(-0.5)
                else:
                    m.setTorque(0)
                    
        elif letter == 'y':
            for i, m in enumerate(motors):
                if i > 19 and i <= 23:
                    m.setPosition(-0.5)
                else:
                    m.setTorque(0)
                    
        elif letter == 'u':
            for i, m in enumerate(motors):
                if i > 23 and i <= 27:
                    m.setPosition(-0.5)
                else:
                    m.setTorque(0)
                    
        elif letter == 'i':
            for i, m in enumerate(motors):
                if i > 27 and i <= 31:
                    m.setPosition(-0.5)
                else:
                    m.setTorque(0)
                    
        elif letter == 'o':
            for i, m in enumerate(motors):
                if i > 31 and i <= 35:
                    m.setPosition(-0.5)
                else:
                    m.setTorque(0)
                    
        elif letter == 'p':
            for i, m in enumerate(motors):
                if i > 35 and i <= 39:
                    m.setPosition(-0.5)
                else:
                    m.setTorque(0)
                    
        elif letter == ' ':
            for i, m in enumerate(motors):
                m.setTorque(0)